@@ -24,8 +24,8 @@ import logging
 
 logger = logging.getLogger('nl')
 here = os.path.join(os.path.dirname(__file__))
-log_dir = here + '/clips/'
-log_file = log_dir + 'log.clp'
+log_dir = os.path.join(here, 'clips')
+log_file = os.path.join(log_dir, 'log.clp')
 if not os.path.isfile(log_file):
     if not os.path.isdir(log_dir):
         os.mkdir(log_dir)
@@ -37,4 +37,9 @@ formatter = logging.Formatter('%(message)s')
 # formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
 hdlr.setFormatter(formatter)
 logger.addHandler(hdlr)
-logger.setLevel(logging.FATAL)
+logger.setLevel(logging.DEBUG)
+
+def hard_debug(self, msg):
+    self.debug(80*'\n'+msg+80*'\n')
+
+setattr(logger, 'hard_debug', hard_debug)
