@@ -1,55 +1,31 @@
 # -*- coding: utf-8 -*-
-'''
-Instant
-    * en una frase -- un num
-    * en una condición -- num / var / arith / now
-    * en una conclusión -- num / var / arith
-
-
-
-# if someone wants to do something, and can do it, she does it
-r1 = Rule([
-        Fact(Person('X1'), Wants(to=State('X4')), Time('X2')),
-        Fact(Person('X1'), Can(what=State('X4')), Time('X3')),
-        During(Time('X2'), Time('X3'))
-        ],[
-        Fact(Person('X1'), State('X4'), Time('X2'))])
-
-def During(inst i, dur d)
-    i > d.start and
-    (not d.end or i < dend)
-
-hay instante y duration. duration tiene instantes start y end.
-una duration siempre tiene start. si no tiene end, se entiende que es futuro.
-la definición de durante es la de arriba.
-se abre y cierra un instante now durante un cierto periodo de interacción con el sistema.
-existe un time 'now' que entra en clips como el now de la linea superior.
-El put action no hace un make instance en una conclusión; de modo que al cerrar una duración, se cierren en todas sus consecuencias.
-
-'''
+# Copyright (c) 2007-2008 by Enrique Pérez Arnaud <enriquepablo@gmail.com>
+#
+# This file is part of ln.
+#
+# ln is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# ln is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with ln.  If not, see <http://www.gnu.org/licenses/>.
 from datetime import datetime
 import clips
 from nl.clps import class_constraint
-from nl.utils import register, subclasses, Name, varpat, clips_instance, _newvar
+from nl import utils
 from nl.arith import Number
 
 _m = []
 
-
-# XXX not thread safe
-_now = str(datetime.now().toordinal())
-
-def start_instant():
-    global _now
-    _now = str(datetime.now().toordinal())
-
-def end_instant():
-    global _now
-    _now = None
-
-
 class Time(Number):
     """
+    abstract ancestor for Instant & Duration
     """
 
     @classmethod
@@ -63,13 +39,28 @@ class Time(Number):
             return Duration.from_clips(instance)
         return Instant(instance)
 
-register('Time', Time)
+utils.register('Time', Time)
 
 class Instant(Time):
+    '''
+    An instant in time.
+    Instantiated with a positive integer or the string 'now'
+
+    >>> i23 = Instant(23)
+    >>> now = Instant('now')
+
+    Can be used as the time attribute in a Fact
+
+    >>> Fact(Thing('X1'), State('X2'), Instant('now'))
+
+    or as start or end attributes in a Duration.
+
+    >>> Duration(start=Instant(1), end=Instant(3))
+    '''
 
     def __init__(self, *args, **kwargs):
         if args and args[0] == 'now':
-            self.value = _now
+            self.value = utils._now
         else:
             super(Instant, self).__init__(*args, **kwargs)
 
@@ -89,7 +80,7 @@ class Instant(Time):
 #
 #'(or (and (eq (class %(ci)s) Duration) (<= (send %(ci)s get-start) %(val)s)) ())'
 
-register('Instant', Instant)
+utils.register('Instant', Instant)
 
 class Duration(Time):
 
@@ -105,11 +96,11 @@ class Duration(Time):
         else:
             self.end = isinstance(end, Instant) and end or \
                                                   Instant(end)
-            if float(self.end.value) == float(_now):
+            if float(self.end.value) == float(utils._now):
                 self.end = Instant('-1.0')
 
     def __str__(self):
-        if varpat.match(self.value):
+        if utils.varpat.match(self.value):
             return self.put_var({})
         else:
             return 'from %s till %s' % (self.start.put({}), self.end.put({}))
@@ -122,8 +113,10 @@ class Duration(Time):
             instance = clips.FindInstance(instance)
         start = Instant(instance.GetSlot('start'))
         if start.value in ('-1', '-1.0', 'now'):
-            start = Instant(_now)
+            start = Instant(utils._now)
         end = Instant(instance.GetSlot('end'))
+        if end.value == 'now':
+            start = Instant('-1.0')
         return Duration(start=start, end=end)
 
     def get_constraint(self, vrs, ancestor, mod_path):
@@ -131,8 +124,8 @@ class Duration(Time):
         build rule CE constraint for clips
         as a mod in a predicate in a prem in a rule
         '''
-        ci = clips_instance(ancestor, mod_path)
-        if varpat.match(self.value):
+        ci = utils.clips_instance(ancestor, mod_path)
+        if utils.varpat.match(self.value):
             return self.get_var_constraint(vrs, ancestor, mod_path, ci)
         else:
             core = '(= (send %s get-start) %s)' % (ci, self.start.get_constraint(vrs))
@@ -145,31 +138,33 @@ class Duration(Time):
         build rule CE slot constraint for clips
         as time in a prem
         """
-        newvar = _newvar()
-        if varpat.match(self.value):
+        newvar = utils._newvar()
+        if utils.varpat.match(self.value):
             return self.get_var_slot_constraint(vrs, self.value)
         core = '(eq (send %s get-start) %s)' % (newvar, self.start.get_slot_constraint(vrs))
         return '?%(var)s&:(and %(core)s (eq (send %(var)s get-end) %(end)s))' % {'core': core, 'var': newvar, 'end': self.end.get_slot_constraint(vrs)}
 
     def put(self, vrs):
-        if varpat.match(self.value):
+        if utils.varpat.match(self.value):
             return self.put_var(vrs)
         else:
-            if getattr(self, 'pstart', False):
-                return '(make-instance of Duration (start %s) (end %s))' % (self.pstart.put(vrs), self.pend.put(vrs))
-            return '(make-instance of Duration (start %s) (end %s))' % (self.start.get_slot_constraint(vrs), self.end.get_slot_constraint(vrs))
+            return '(make-instance of Duration (start %s) (end %s))' % \
+                (getattr(self, 'pstart', False) and self.pstart.put(vrs) or \
+                                           self.start.get_slot_constraint(vrs),
+                 getattr(self, 'pend', False) and self.pend.put(vrs) or \
+                                           self.end.get_slot_constraint(vrs))
 
     def get_isc(self, templs, queries, vrs):
         """
         get instance-set condition;
         modify (instance-set templates, instance-set queries)
         """
-        newvar = _newvar()
-        if varpat.match(self.value):
+        newvar = utils._newvar()
+        if utils.varpat.match(self.value):
             if self.value in vrs:
                 if vrs[self.value]:
                     queries.append('(eq ?%s %s)' % (newvar,
-                                     clips_instance(*(vrs[self.value]))))
+                                     utils.clips_instance(*(vrs[self.value]))))
                 else:
                     newvar = self.value
             else:
@@ -177,46 +172,54 @@ class Duration(Time):
                 newvar = self.value
         templs.append((newvar, 'Duration'))
         start = getattr(self, 'start', _m)
-        if start is not _m and not (varpat.match(start.value) and start.value not in vrs):
+        if utils.varpat.match(self.value):
+            return '?%s' % newvar
+        if start is not _m and not (utils.varpat.match(start.value) and start.value not in vrs):
             queries.append('(= ?%s:start %s)' % (newvar,
                                            start.get_isc(templs, queries, vrs)))
         end = getattr(self, 'end', _m)
-        if end is not _m and not (varpat.match(end.value) and end.value not in vrs):
+        if end is not _m and not (utils.varpat.match(end.value) and end.value not in vrs):
             queries.append('(= ?%s:end %s)' % (newvar,
                                            end.get_isc(templs, queries, vrs)))
         return '?%s' % newvar
 
-register('Duration', Duration)
+utils.register('Duration', Duration)
 
-class Finish(Name):
-    def __init__(self, duration):
+class Finish(utils.Name):
+    def __init__(self, duration, instant):
         self.duration = isinstance(duration, Duration) and \
                                 duration or Duration(duration)
+        self.instant = isinstance(instant, Instant) and \
+                                instant or Instant(instant)
 
     def put_action(self, vrs):
-        return '(send %s put-end %s)' % (self.duration.put(vrs), _now)
+        return '(modify-instance %s (end %s))' % (self.duration.put(vrs), self.instant.put(vrs))
 
-register('Finish', Finish)
+utils.register('Finish', Finish)
 
 
-class During(Name):
+class During(utils.Name):
     '''
     given an instant and a duration, build a condition for a rule
     that tests whether the instant is within the duration
     '''
-    def __init__(self, instant, duration):
+    def __init__(self, instant, *durations):
         self.instant = isinstance(instant, Instant) and \
                                    instant or Instant(instant)
-        self.duration = isinstance(duration, str) and \
+        self.durations = [isinstance(duration, str) and \
                                    Duration(duration) or duration
+                                         for duration in durations]
 
     def get_ce(self, vrs):
-        return '(test (and (<= (send %(dur)s get-start) %(ins)s) (or (= (send %(dur)s get-end) -1) (>= (send %(dur)s get-end) %(ins)s))))' % {'dur': self.duration.put(vrs), 'ins': self.instant.put(vrs)}
+        durs = []
+        for duration in self.durations:
+            durs.append('(test (and (<= (send %(dur)s get-start) %(ins)s) (or (= (send %(dur)s get-end) -1) (>= (send %(dur)s get-end) %(ins)s))))' % {'dur': duration.put(vrs), 'ins': self.instant.put(vrs)})
+        return ' '.join(durs)
 
-register('During', During)
+utils.register('During', During)
 
 
-class DurationOpMixin(Name):
+class DurationOpMixin(utils.Name):
     '''
     Abstract ancestor of classes constructed with a sequence of durations
     '''
@@ -225,7 +228,7 @@ class DurationOpMixin(Name):
           [isinstance(dur, Duration) and dur or Duration(dur) for dur in args]
 
 
-register('DurationOpMixin', DurationOpMixin)
+utils.register('DurationOpMixin', DurationOpMixin)
 
 
 class Coincide(DurationOpMixin):
@@ -243,7 +246,7 @@ class Coincide(DurationOpMixin):
                 """ % {'durs': ' '.join([dur.put(vrs) for dur in self.durations])}
 
 
-register('Coincide', Coincide)
+utils.register('Coincide', Coincide)
 
 class Intersection(DurationOpMixin):
     '''
@@ -258,7 +261,7 @@ class Intersection(DurationOpMixin):
                                            (end (maxcomend %(durs)s)))
                 """ % {'durs': ' '.join([dur.put(vrs) for dur in self.durations])}
 
-register('Intersection', Intersection)
+utils.register('Intersection', Intersection)
 
 class MinComStart(DurationOpMixin):
     """
@@ -269,7 +272,7 @@ class MinComStart(DurationOpMixin):
         instants = [dur.put(vrs) for dur in self.durations]
         return '(mincomstart %s)' % ' '.join(instants)
 
-register('MinComStart', MinComStart)
+utils.register('MinComStart', MinComStart)
 
 class MaxComEnd(DurationOpMixin):
     """
@@ -280,4 +283,4 @@ class MaxComEnd(DurationOpMixin):
         instants = [dur.put(vrs) for dur in self.durations]
         return '(maxcomend %s)' % ' '.join(instants)
 
-register('MaxComEnd', MaxComEnd)
+utils.register('MaxComEnd', MaxComEnd)
